import logging

from sais.notify.auth.auth_info import EnvVarCredentialsProvider
from sais.notify.clients.notify_client import NotifyClient
from sais.notify.config import const
from sais.notify.config.const import ENDPOINT, NOTIFY_SERVICE_NOTIFY_TYPE_FEISHU, \
    NOTIFY_SERVICE_NOTIFY_TYPE_FEISHU_GROUP, LOGGER_NAME, NOTIFY_SERVICE_NOTIFY_TYPE_EMAIL
from sais.notify.model.message_model import NotificationRequest, MessageModel
from sais.notify.types import NotifyType

logger = logging.getLogger(LOGGER_NAME)
logging.basicConfig(level=logging.INFO)


class MessageHandler(object):
    def __init__(self, auth_provider: EnvVarCredentialsProvider):
        self.auth_provider = auth_provider

    def send_notification(self, notify_type: NotifyType, to: str, message: str, subject: str = None) -> bool:
        """
        发送通知的函数。

        参数:
        - notify_type: 通知类型，枚举类型NotifyType指定。
        - to: 接收通知的目标标识，可以是用户或群组的标识。
        - message: 要发送的消息内容。

        返回值:
        - bool: 发送成功返回True，失败返回False。
        """
        logger.info(f'info send notification: {notify_type}, {to}, {message}')
        logger.debug(f'debug send notification: {notify_type}, {to}, {message}')
        request = NotificationRequest(notify_type=notify_type, to=to, message=message)
        request.model_dump()
        if notify_type in (NotifyType.FEISHU_USER_TEXT, NotifyType.FEISHU_USER_RICH_TEXT,
                           NotifyType.FEISHU_GROUP_TEXT, NotifyType.FEISHU_GROUP_MESSAGE_CARD):
            return self.__send_notification_feishu(notify_type, to, message)
        elif notify_type == NotifyType.EMAIL:
            return self.__send_notification_email(to, message, subject)
        else:
            logger.error(f'not support notify type: {notify_type}')
        return False

    def __send_notification_feishu(self, notify_type: NotifyType, to: str, message: str) -> bool:
        client = NotifyClient(ENDPOINT, self.auth_provider)
        message_model = MessageModel(
            message=message,
            to=to,
            type={
                NotifyType.FEISHU_USER_TEXT: NOTIFY_SERVICE_NOTIFY_TYPE_FEISHU,
                NotifyType.FEISHU_USER_RICH_TEXT: NOTIFY_SERVICE_NOTIFY_TYPE_FEISHU,
                NotifyType.FEISHU_GROUP_TEXT: NOTIFY_SERVICE_NOTIFY_TYPE_FEISHU_GROUP,
                NotifyType.FEISHU_GROUP_MESSAGE_CARD: NOTIFY_SERVICE_NOTIFY_TYPE_FEISHU_GROUP,
            }.get(notify_type, None)
        )

        robot_name = const.SAIS_ROBOT_NAME if notify_type == NotifyType.FEISHU_USER_RICH_TEXT else None
        message_model.robot_name = robot_name

        if message_model.type is None:
            logger.error(f'not support notify type: {notify_type}')
            return False

        return client.send_notification(message_model)

    def __send_notification_email(self, to: str, message: str, subject: str) -> bool:
        client = NotifyClient(ENDPOINT, self.auth_provider)
        message_model = MessageModel(
            subject=subject,
            message=message,
            to=to,
            type=NOTIFY_SERVICE_NOTIFY_TYPE_EMAIL
        )


        return client.send_notification(message_model)


from enum import Enum


class MyStrEnum(Enum):
    def __new__(cls, value, member_name=None):
        obj = object.__new__(cls)
        obj._value_ = value
        obj.member_name = member_name or value
        return obj

    def __str__(self):
        return self.value


class NotifyType(MyStrEnum):
    FEISHU_USER_TEXT = "feishu_user_text"
    FEISHU_USER_RICH_TEXT = "feishu_user_rich_text"
    FEISHU_GROUP_TEXT = "feishu_group_text"
    FEISHU_GROUP_MESSAGE_CARD = "feishu_group_message_card"
    EMAIL = "email"



ENDPOINT = 'http://fuxi-ai.cn:10010'
NOTIFY_AUTH_TOKEN = 'SAIS_NOTIFY_AUTH_TOKEN'

SAIS_ROBOT_NAME = 'sais-post-robot'
NOTIFY_SERVICE_NOTIFY_TYPE_FEISHU = 'feishu'
NOTIFY_SERVICE_NOTIFY_TYPE_FEISHU_GROUP = 'feishu_group'
NOTIFY_SERVICE_NOTIFY_TYPE_EMAIL = 'email'
LOGGER_NAME = 'sais'



import json
import logging
from http import HTTPStatus

import requests

from sais.notify.auth.auth_info import EnvVarCredentialsProvider
from sais.notify.config.const import LOGGER_NAME
from sais.notify.model.message_model import MessageModel

logger = logging.getLogger(LOGGER_NAME)
logging.basicConfig(level=logging.INFO)


class NotifyClient(object):
    def __init__(self, endpoint, auth_provider: EnvVarCredentialsProvider):
        self.endpoint = endpoint
        self.auth_provider = auth_provider

    def send_notification(self, message_info: MessageModel) -> bool:
        headers = {
            'Authorization': self.auth_provider.token,
            'Content-Type': 'application/json'
        }
        response = requests.post(f'{self.endpoint}/api/v1/notify/send', headers=headers,
                                 json=message_info.model_dump(by_alias=True, exclude_none=True))

        if response.status_code == HTTPStatus.OK:
            response_data = response.json()
            if response_data.get('msgCode') == '10000':
                logger.info(f'Notification sent type {message_info.type} to {message_info.to} successfully')
                return True
            else:
                logger.error(f'Error sending notification: {response.status_code} - {response.text}')
        else:
            logger.error(f'Error sending notification: {response.status_code}')
        return False
